"""
utils for nsedt
"""

import json
import datetime
from warnings import warn
import pandas as pd
import requests
from nsedt.resources import constants as cns



def get_headers():
    """
    Args:
        ---\n
    Returns:\n
        Json: json containing nse header
    """

    return {
        "Host": "www.nseindia.com",
        "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:85.0) Gecko/20100101 Firefox/85.0",
        "Accept": "*/*",
        "Accept-Language": "en-US,en;q=0.5",
        "Accept-Encoding": "gzip, deflate, br",
        "X-Requested-With": "XMLHttpRequest",
        "DNT": "1",
        "Connection": "keep-alive",
    }


def get_cookies():
    """
    Args:
        ---

    Returns:

        Json: json containing nse cookies
    """

    response = requests.get(cns.BASE_URL, timeout=30, headers=get_headers())
    if response.status_code != 200:
        raise ValueError("Retry again in a minute.")
    return response.cookies.get_dict()


def fetch_url(url, cookies, key=None, response_type="panda_df"):
    """
    Args:

        url (str): URL to fetch
        cookies (str): NSE cookies
        key (str, Optional):

    Returns:

        Pandas DataFrame: df containing url data

    """

    response = requests.get(
        url=url,
        timeout=30,
        headers=get_headers(),
        cookies=cookies,
    )

    if response.status_code == 200:
        json_response = json.loads(response.content)

        if response_type != "panda_df":
            return json_response
        if key is None:
            return pd.DataFrame.from_dict(json_response)

        return pd.DataFrame.from_dict(json_response[key])

    raise ValueError("Please try again in a minute.")


def get_symbol(symbol: str, get_key: str) -> str:
    """_summary_

    Args:
        symbol (str): _description_
        get_key (str): _description_

    Returns:
        str: _description_
    """

    symbol_map = cns.SYMBOL_MAP
    val = None
    for item in symbol_map:
        key_list = item["keys"]
        if symbol in key_list:
            val = item[get_key]

    return val if val else symbol


def check_nd_convert(start_date: str, end_date: str) -> datetime:
    """
    The function `check_nd_convert` takes two date strings in the format "%d-%m-%Y" and 
    converts them to datetime objects if they are not already in that format.

    :param start_date: The `start_date` parameter is the starting date of a period,
     specified as a string in the format "%d-%m-%Y"
    :type start_date: str
    :param end_date: The `end_date` parameter is a string representing the end date in the format
    "%d-%m-%Y"
    :type end_date: str
    :return: the start_date and end_date as datetime objects.
    """

    if isinstance(start_date, datetime.date) and isinstance(end_date, datetime.date):
        warn(
            """Passing start_date, end_date in date is deprecated
now pass in str '%d-%m-%Y' format""",
            DeprecationWarning,
            stacklevel=2,
        )

    elif isinstance(start_date, str) and isinstance(end_date, str):
        start_date = datetime.datetime.strptime(start_date, "%d-%m-%Y")
        end_date = datetime.datetime.strptime(end_date, "%d-%m-%Y")

    else:
        raise ValueError("Input is of an unknown type")

    return start_date, end_date



""" 
get data for Options
"""
import logging
import urllib

from datetime import datetime
from nsedt import utils
from nsedt.resources import constants as cns
from nsedt.utils import data_format

log = logging.getLogger("root")


def get_option_chain(
    symbol: str,
    strike_price: str = None,
    expiry_date: str = None,
    response_type="panda_df",
):
    """Get option data of stock and indices

    Args:
        symbol (str): symbol name
        strike_price (str, optional): strike price to apply filter on price. Defaults to None.
        expiry_date (str, optional): expiry date to apply filter on date. Defaults to None.
        response_type (str, optional): response_type panda_df or json . Defaults to "panda_df".

    Returns:
        Pandas DataFrame: df containing option data
    or
        Json: json containing option data

    """
    params = {}
    cookies = utils.get_cookies()
    base_url = cns.BASE_URL
    symbol = utils.get_symbol(symbol=symbol, get_key="derivatives")

    if symbol in cns.INDICES:
        event_api = cns.OPTIONS_PRICE_INDICES
    else:
        event_api = cns.OPTIONS_PRICE_EQUITIES

    params["symbol"] = symbol

    url = base_url + event_api + urllib.parse.urlencode(params)
    data = utils.fetch_url(url, cookies, response_type="json")

    if data is None or data == {}:
        log.error("symbol is wrong or unable to access API")
        raise ValueError

    # filtering data

    if strike_price and expiry_date:
        filtered_data = [
            record
            for record in data["records"]["data"]
            if record["strikePrice"] == strike_price
            and record["expiryDate"]
            == datetime.strptime(expiry_date, "%d-%m-%Y").strftime("%d-%b-%Y")
        ]

    elif strike_price:
        filtered_data = [
            record
            for record in data["records"]["data"]
            if record["strikePrice"] == strike_price
        ]
    elif expiry_date:
        filtered_data = [
            record
            for record in data["records"]["data"]
            if record["expiryDate"]
            == datetime.strptime(expiry_date, "%d-%m-%Y").strftime("%d-%b-%Y")
        ]
    else:
        filtered_data = data["records"]["data"]

    return data_format.option_chain(
        filtered_data,
        response_type=response_type,
    )


def get_option_chain_expdate(symbol: str) -> list:
    """get option  expiry date for stock and indices

    Args:
        symbol (str): symbol name

    Returns:
        list: expiry date in list("%d-%m-%Y" format)
    """
    params = {}
    cookies = utils.get_cookies()
    base_url = cns.BASE_URL

    if symbol in cns.INDICES:
        event_api = cns.OPTIONS_PRICE_INDICES
    else:
        event_api = cns.OPTIONS_PRICE_EQUITIES

    params["symbol"] = symbol

    url = base_url + event_api + urllib.parse.urlencode(params)
    data = utils.fetch_url(url, cookies, response_type="json")
    ret = []
    expiry_dates = data.get("records").get("expiryDates")
    if expiry_dates is None:
        log.error("expiry_dates is None, symbol is wrong or unable to access API")
        return []
    for expiry_date in expiry_dates:
        ret.append(datetime.strptime(expiry_date, "%d-%b-%Y").strftime("%d-%m-%Y"))
    return ret



"""
    NSE indices symbol map
"""
symbol_map = [
    {
        "keys": ["NIFTY 50", "NIFTY50", "NIFTY"],
        "indices": "NIFTY 50",
        "derivatives": "NIFTY",
    },
    {
        "keys": ["NIFTY BANK", "NIFTYBANK", "BANKNIFTY"],
        "derivatives": "BANKNIFTY",
        "indices": "NIFTY BANK",
    },
    {
        "keys": ["NIFTY FINANCIAL SERVICES", "FINNIFTY", "NIFTY FIN SERVICE"],
        "derivatives": "FINNIFTY",
        "indices": "NIFTY FINANCIAL SERVICES",
    },
    {"keys": ["NIFTY NEXT 50"], "derivatives": "", "indices": "NIFTY NEXT 50"},
    {"keys": ["NIFTY MIDCAP 50"], "derivatives": "", "indices": "NIFTY MIDCAP 50"},
    {"keys": ["NIFTY MIDCAP 100"], "derivatives": "", "indices": "NIFTY MIDCAP 100"},
    {"keys": ["NIFTY MIDCAP 150"], "derivatives": "", "indices": "NIFTY MIDCAP 150"},
    {"keys": ["NIFTY SMALLCAP 50"], "derivatives": "", "indices": "NIFTY SMALLCAP 50"},
    {
        "keys": ["NIFTY SMALLCAP 100"],
        "derivatives": "",
        "indices": "NIFTY SMALLCAP 100",
    },
    {
        "keys": ["NIFTY SMALLCAP 250"],
        "derivatives": "",
        "indices": "NIFTY SMALLCAP 250",
    },
    {
        "keys": ["NIFTY MIDSMALLCAP 400"],
        "derivatives": "",
        "indices": "NIFTY MIDSMALLCAP 400",
    },
    {"keys": ["NIFTY 100"], "derivatives": "", "indices": "NIFTY 100"},
    {"keys": ["NIFTY 200"], "derivatives": "", "indices": "NIFTY 200"},
    {
        "keys": ["NIFTY500 MULTICAP 50:25:25"],
        "derivatives": "",
        "indices": "NIFTY500 MULTICAP 50:25:25",
    },
    {
        "keys": ["NIFTY LARGEMIDCAP 250"],
        "derivatives": "",
        "indices": "NIFTY LARGEMIDCAP 250",
    },
    {
        "keys": ["NIFTY MIDCAP SELECT"],
        "derivatives": "",
        "indices": "NIFTY MIDCAP SELECT",
    },
    {
        "keys": ["NIFTY TOTAL MARKET"],
        "derivatives": "",
        "indices": "NIFTY TOTAL MARKET",
    },
    {
        "keys": ["NIFTY MICROCAP 250"],
        "derivatives": "",
        "indices": "NIFTY MICROCAP 250",
    },
    {"keys": ["NIFTY AUTO"], "derivatives": "", "indices": "NIFTY AUTO"},
    {"keys": ["NIFTY ENERGY"], "derivatives": "", "indices": "NIFTY ENERGY"},
    {
        "keys": ["NIFTY FINANCIAL SERVICES 25/50"],
        "derivatives": "",
        "indices": "NIFTY FINANCIAL SERVICES 25/50",
    },
    {"keys": ["NIFTY FMCG"], "derivatives": "", "indices": "NIFTY FMCG"},
    {"keys": ["NIFTY IT"], "derivatives": "", "indices": "NIFTY IT"},
    {"keys": ["NIFTY MEDIA"], "derivatives": "", "indices": "NIFTY MEDIA"},
    {"keys": ["NIFTY METAL"], "derivatives": "", "indices": "NIFTY METAL"},
    {"keys": ["NIFTY PHARMA"], "derivatives": "", "indices": "NIFTY PHARMA"},
    {"keys": ["NIFTY PSU BANK"], "derivatives": "", "indices": "NIFTY PSU BANK"},
    {"keys": ["NIFTY REALTY"], "derivatives": "", "indices": "NIFTY REALTY"},
    {
        "keys": ["NIFTY PRIVATE BANK"],
        "derivatives": "",
        "indices": "NIFTY PRIVATE BANK",
    },
    {
        "keys": ["NIFTY HEALTHCARE INDEX"],
        "derivatives": "",
        "indices": "NIFTY HEALTHCARE INDEX",
    },
    {
        "keys": ["NIFTY CONSUMER DURABLES"],
        "derivatives": "",
        "indices": "NIFTY CONSUMER DURABLES",
    },
    {
        "keys": ["NIFTY OIL &amp; GAS"],
        "derivatives": "",
        "indices": "NIFTY OIL &amp; GAS",
    },
    {
        "keys": ["NIFTY DIVIDEND OPPORTUNITIES 50"],
        "derivatives": "",
        "indices": "NIFTY DIVIDEND OPPORTUNITIES 50",
    },
    {"keys": ["NIFTY50 VALUE 20"], "derivatives": "", "indices": "NIFTY50 VALUE 20"},
    {
        "keys": ["NIFTY100 QUALITY 30"],
        "derivatives": "",
        "indices": "NIFTY100 QUALITY 30",
    },
    {
        "keys": ["NIFTY50 EQUAL WEIGHT"],
        "derivatives": "",
        "indices": "NIFTY50 EQUAL WEIGHT",
    },
    {
        "keys": ["NIFTY100 EQUAL WEIGHT"],
        "derivatives": "",
        "indices": "NIFTY100 EQUAL WEIGHT",
    },
    {
        "keys": ["NIFTY100 LOW VOLATILITY 30"],
        "derivatives": "",
        "indices": "NIFTY100 LOW VOLATILITY 30",
    },
    {"keys": ["NIFTY ALPHA 50"], "derivatives": "", "indices": "NIFTY ALPHA 50"},
    {
        "keys": ["NIFTY200 QUALITY 30"],
        "derivatives": "",
        "indices": "NIFTY200 QUALITY 30",
    },
    {
        "keys": ["NIFTY ALPHA LOW-VOLATILITY 30"],
        "derivatives": "",
        "indices": "NIFTY ALPHA LOW-VOLATILITY 30",
    },
    {
        "keys": ["NIFTY200 MOMENTUM 30"],
        "derivatives": "",
        "indices": "NIFTY200 MOMENTUM 30",
    },
    {
        "keys": ["NIFTY MIDCAP150 QUALITY 50"],
        "derivatives": "",
        "indices": "NIFTY MIDCAP150 QUALITY 50",
    },
    {"keys": ["NIFTY COMMODITIES"], "derivatives": "", "indices": "NIFTY COMMODITIES"},
    {
        "keys": ["NIFTY INDIA CONSUMPTION"],
        "derivatives": "",
        "indices": "NIFTY INDIA CONSUMPTION",
    },
    {"keys": ["NIFTY CPSE"], "derivatives": "", "indices": "NIFTY CPSE"},
    {
        "keys": ["NIFTY INFRASTRUCTURE"],
        "derivatives": "",
        "indices": "NIFTY INFRASTRUCTURE",
    },
    {"keys": ["NIFTY MNC"], "derivatives": "", "indices": "NIFTY MNC"},
    {
        "keys": ["NIFTY GROWTH SECTORS 15"],
        "derivatives": "",
        "indices": "NIFTY GROWTH SECTORS 15",
    },
    {"keys": ["NIFTY PSE"], "derivatives": "", "indices": "NIFTY PSE"},
    {
        "keys": ["NIFTY SERVICES SECTOR"],
        "derivatives": "",
        "indices": "NIFTY SERVICES SECTOR",
    },
    {
        "keys": ["NIFTY100 LIQUID 15"],
        "derivatives": "",
        "indices": "NIFTY100 LIQUID 15",
    },
    {
        "keys": ["NIFTY MIDCAP LIQUID 15"],
        "derivatives": "",
        "indices": "NIFTY MIDCAP LIQUID 15",
    },
    {
        "keys": ["NIFTY INDIA DIGITAL"],
        "derivatives": "",
        "indices": "NIFTY INDIA DIGITAL",
    },
    {"keys": ["NIFTY100 ESG"], "derivatives": "", "indices": "NIFTY100 ESG"},
    {
        "keys": ["NIFTY INDIA MANUFACTURING"],
        "derivatives": "",
        "indices": "NIFTY INDIA MANUFACTURING",
    },
]



""" 
get data for Futures
"""
import urllib
from datetime import datetime, timedelta

from nsedt import utils
from nsedt.resources import constants as cns
from nsedt.utils import data_format


def get_future_price(
    symbol: str,
    start_date: str,
    end_date: str,
    expiry_date: str = None,
    response_type: str = "panda_df",
    columns_drop_list: list = None,
):
    """
        get future price of stock / indices
    Args:
        symbol (str): _description_
        start_date (str): _description_
        end_date (str): _description_
        expiry_date (str, optional): _description_. Defaults to None.
        response_type (str, optional): _description_. Defaults to "panda_df".
        columns_drop_list (list, optional): _description_. Defaults to None.

    Returns:
        _type_: _description_
    """
    cookies = utils.get_cookies()
    base_url = cns.BASE_URL
    event_api = cns.FUTURES_PRICE
    symbol = utils.get_symbol(symbol=symbol, get_key="derivatives")
    params = {
        "symbol": symbol,
        "from": start_date,
        "to": end_date,
    }
    if symbol in cns.INDICES:
        params["instrumentType"] = "FUTIDX"
    else:
        params["instrumentType"] = "FUTSTK"

    url = base_url + event_api + urllib.parse.urlencode(params)
    data = utils.fetch_url(url, cookies, response_type="json")

    if expiry_date:
        filtered_data = [
            record
            for record in data["data"]
            if record["FH_EXPIRY_DT"]
            == datetime.strptime(expiry_date, "%d-%m-%Y").strftime("%d-%b-%Y")
        ]
    else:
        filtered_data = data["data"]

    return data_format.derivatives_futures(
        filtered_data,
        response_type=response_type,
        columns_drop_list=columns_drop_list,
    )


def get_future_expdate(symbol: str) -> list:
    """get expiry dates of futures

    Args:
        symbol (str): symbol name

    Returns:
        list: expiry dates
    """
    cookies = utils.get_cookies()
    base_url = cns.BASE_URL
    event_api = cns.FUTURES_PRICE
    params = {
        "symbol": symbol,
        "from": (datetime.now() - timedelta(days=3)).strftime("%d-%m-%Y"),
        "to": datetime.now().strftime("%d-%m-%Y"),
    }
    if symbol in cns.INDICES:
        params["instrumentType"] = "FUTIDX"
    else:
        params["instrumentType"] = "FUTSTK"

    url = base_url + event_api + urllib.parse.urlencode(params)
    data = utils.fetch_url(url, cookies, response_type="json")
    ret = []
    for rec in data["data"]:
        ret.append(
            datetime.strptime(rec["FH_EXPIRY_DT"], "%d-%b-%Y").strftime("%d-%m-%Y")
        )
    return list(set(ret))



""" 
get data for Equity
"""
import concurrent
import logging
import urllib
from concurrent.futures import ALL_COMPLETED
from datetime import datetime, timedelta

import pandas as pd
from nsedt import utils
from nsedt.resources import constants as cns
from nsedt.utils import data_format
from nsedt.derivatives.options import get_option_chain, get_option_chain_expdate
from nsedt.derivatives.futures import get_future_price, get_future_expdate

log = logging.getLogger("root")


def get_vix(
    start_date: str,
    end_date: str,
    response_type: str = "panda_df",
    columns_drop_list: list = None,
):
    """Get Vix data

    Args:
        start_date (str): start date in "%d-%m-%Y" format
        end_date (str): end_date in "%d-%m-%Y" format
        response_type (str, optional): response_type. Defaults to "panda_df".
        columns_drop_list (list, optional): provide custom columns drop list. Defaults to None.

    Raises:
        exc: general Exception

    Returns:
            Pandas DataFrame: df containing option data
        or
            Json: json containing option data

    """
    cookies = utils.get_cookies()
    params = {
        "from": start_date,
        "to": end_date,
    }
    base_url = cns.BASE_URL
    event_api = cns.VIX_HISTORY
    url_list = []
    start_date = datetime.strptime(start_date, "%d-%m-%Y")
    end_date = datetime.strptime(end_date, "%d-%m-%Y")

    # set the window size to one year
    window_size = timedelta(days=cns.WINDOW_SIZE)

    current_window_start = start_date
    while current_window_start < end_date:
        current_window_end = current_window_start + window_size

        # check if the current window extends beyond the end_date
        current_window_end = min(current_window_end, end_date)

        params = {
            "from": current_window_start.strftime("%d-%m-%Y"),
            "to": current_window_end.strftime("%d-%m-%Y"),
        }
        url = base_url + event_api + urllib.parse.urlencode(params)
        url_list.append(url)

        # move the window start to the next day after the current window end
        current_window_start = current_window_end + timedelta(days=1)

    result = pd.DataFrame()
    with concurrent.futures.ThreadPoolExecutor(max_workers=cns.MAX_WORKERS) as executor:
        future_to_url = {
            executor.submit(utils.fetch_url, url, cookies, response_type="panda"): url
            for url in url_list
        }
        concurrent.futures.wait(future_to_url, return_when=ALL_COMPLETED)
        for future in concurrent.futures.as_completed(future_to_url):
            url = future_to_url[future]
            try:
                dataframe = data_format.get_vix(
                    future.result(),
                    response_type=response_type,
                    columns_drop_list=columns_drop_list,
                )
                result = pd.concat([result, dataframe])
            except Exception as exc:
                log.error("%s got exception: %s. Please try again later.", url, exc)
                raise exc

    if response_type == "panda_df":
        return result
    return result.to_json(orient="records")



"""
return data in specific format
"""

import pandas as pd


def price(result):
    """
    Args:
        result (Pandas DataFrame): result

    Returns:
        Pandas DataFrame: df containing data in specific format
    """

    columns_required = [
        "CH_TIMESTAMP",
        "CH_OPENING_PRICE",
        "CH_TRADE_HIGH_PRICE",
        "CH_TRADE_LOW_PRICE",
        "CH_CLOSING_PRICE",
        "CH_PREVIOUS_CLS_PRICE",
        "CH_TOT_TRADED_QTY",
        "CH_TOT_TRADED_VAL",
        "CH_52WEEK_HIGH_PRICE",
        "CH_52WEEK_LOW_PRICE",
        "VWAP",
        "COP_DELIV_QTY",
        "COP_DELIV_PERC",
        "CH_SERIES",
    ]
    try:
        result = result[columns_required]
    except:  # pylint: disable=W0702
        return result
    result = result.set_axis(
        [
            "Date",
            "Open Price",
            "High Price",
            "Low Price",
            "Close Price",
            "Prev Close Price",
            "Total Traded Quantity",
            "Total Traded Value",
            "52 Week High Price",
            "52 Week Low Price",
            "VWAP",
            "Deliverable Volume",
            "Deliverable Percent",
            "Series",
        ],
        axis=1,
    )

    result["Date"] = pd.to_datetime(result["Date"])
    result = result.sort_values("Date", ascending=True)
    result.reset_index(drop=True, inplace=True)
    return result


def indices(
    data_json,
    columns_drop_list: list = None,
    columns_rename_map: map = None,
):
    """
    Args:
        data_json (json):  data in json format
    Returns:
        Pandas DataFrame: df with indexCloseOnlineRecords and indexTurnoverRecords
    """
    if columns_drop_list:
        columns_list = columns_drop_list
    else:
        columns_list = ["_id", "EOD_INDEX_NAME", "TIMESTAMP"]

    if columns_rename_map:
        columns_rename = columns_rename_map
    else:
        columns_rename = {
            "EOD_OPEN_INDEX_VAL": "Open Price",
            "EOD_HIGH_INDEX_VAL": "High Price",
            "EOD_CLOSE_INDEX_VAL": "Close Price",
            "EOD_LOW_INDEX_VAL": "Low Price",
            "EOD_TIMESTAMP": "Date",
        }
    data_close_df = (
        pd.DataFrame(data_json["data"]["indexCloseOnlineRecords"])
        .drop(columns=columns_list)
        .rename(columns=columns_rename)
    )

    ## Mismatch values
    # data_turnover_df = (
    #     pd.DataFrame(d["data"]["indexTurnoverRecords"])
    #     .drop(columns=["_id", "HIT_INDEX_NAME_UPPER", "HIT_TIMESTAMP"])
    #     .rename(
    #         columns={
    #             "HIT_TRADED_QTY": "Total Traded Quantity",
    #             "HIT_TURN_OVER": "Total Traded Value",
    #             "TIMESTAMP": "Date",
    #         }
    #     )
    # )
    return data_close_df
    # return pd.merge(data_close_df, data_turnover_df, on="Date", how="inner")


def option_chain(
    data_json: str,
    response_type: str,
):
    """_summary_

    Args:
        data_json (str): _description_
        response_type (str): _description_

    Returns:
        _type_: _description_
    """
    if response_type == "json":
        data_json_ret = []
        for record in data_json:
            if "PE" in record:
                record["PE"].pop("strikePrice", None)
                record["PE"].pop("expiryDate", None)
                record["PE"].pop("underlying", None)
                record["PE"].pop("identifier", None)
            if "CE" in record:
                record["CE"].pop("strikePrice", None)
                record["CE"].pop("expiryDate", None)
                record["CE"].pop("underlying", None)
                record["CE"].pop("identifier", None)
            data_json_ret.append(record)
        return data_json_ret

    return (
        pd.json_normalize(data_json)
        .sort_values(by=["expiryDate", "strikePrice"], ascending=True)
        .drop(
            columns=[
                "PE.strikePrice",
                "PE.expiryDate",
                "PE.identifier",
                "CE.strikePrice",
                "CE.expiryDate",
                "CE.identifier",
            ]
        )
    )


def get_vix(
    data_json: object,
    response_type: str = "panda_df",
    columns_drop_list: list = None,
):
    """
        Format Vix data
    Args:
        data_json (object): data in json format.
        response_type (str, optional): response_type. Defaults to "panda_df".
        columns_drop_list (list, optional): custom columns drop list. Defaults to None.
    Returns:
        _type_: _description_
    """
    data_json = data_json["data"]
    if columns_drop_list:
        columns_list = columns_drop_list
    else:
        columns_list = [
            "_id",
            "TIMESTAMP",
            "createdAt",
            "updatedAt",
            "__v",
            "ALTERNATE_INDEX_NAME",
            "EOD_INDEX_NAME",
            "EOD_PREV_CLOSE",
            "VIX_PTS_CHG",
            "VIX_PERC_CHG",
        ]
    if response_type == "json":
        data_json_ret = []
        for record in data_json:
            for column in columns_list:
                record.pop(column, None)

            data_json_ret.append(record)
        return data_json_ret

    return (
        pd.json_normalize(data_json)
        .drop(columns=columns_list)
        .rename(
            columns={
                "EOD_OPEN_INDEX_VAL": "Open Price",
                "EOD_HIGH_INDEX_VAL": "High Price",
                "EOD_CLOSE_INDEX_VAL": "Close Price",
                "EOD_LOW_INDEX_VAL": "Low Price",
                "EOD_TIMESTAMP": "Date",
            }
        )
    )


def derivatives_futures(
    data_json: str,
    response_type: str = "panda_df",
    columns_drop_list=None,
):
    """
        Format futures data

    Args:
        data_json (object): data in json format.
        response_type (str, optional): response_type. Defaults to "panda_df".
        columns_drop_list (list, optional): custom columns drop list. Defaults to None.

    Returns:
            json: formate data in json
        or
            dataframe: formate data in panda df
    """
    if columns_drop_list:
        columns_list = columns_drop_list
    else:
        columns_list = [
            "_id",
            "FH_MARKET_LOT",
            "FH_MARKET_TYPE",
            "FH_OPTION_TYPE",
            "FH_SYMBOL",
            "FH_INSTRUMENT",
            "FH_STRIKE_PRICE",
            "FH_LAST_TRADED_PRICE",
            "TIMESTAMP",
        ]
    if response_type == "json":
        data_json_ret = []
        for record in data_json:
            for column in columns_list:
                record.pop(column, None)
            data_json_ret.append(record)
        return data_json_ret

    return (
        pd.json_normalize(data_json)
        .drop(columns=columns_list)
        .rename(
            columns={
                "FH_OPENING_PRICE": "Open Price",
                "FH_TRADE_HIGH_PRICE": "High Price",
                "FH_CLOSING_PRICE": "Close Price",
                "FH_TRADE_LOW_PRICE": "Low Price",
                "FH_CHANGE_IN_OI": "Change in OI",
                "FH_EXPIRY_DT": "Expiry Date",
                "FH_TIMESTAMP": "Date",
            }
        )
    )
